<?php

namespace Drupal\reviews\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\node\NodeInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Controller for our dynamic tab.
 */
class ReviewsController extends ControllerBase {

  /**
   * Route title callback.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node entity.
   *
   * @return string
   *   The title.
   */
  public function getReviewsPageTitle(NodeInterface $node) {
    // Get review entities for the given node.
    $query = \Drupal::entityQuery('review')
      ->condition('node_id', $node->id(), '=')
      ->condition('status', 1)
      ->accessCheck(FALSE)
      ->execute();

    $review_count = count($query);

    return $this->t('Reviews (@review_count)', ['@review_count' => $review_count]);
  }

  /**
   * Returns a renderable array for the reviews page of a node.
   */
  public function reviews(NodeInterface $node) {
    // Get the config for the reviews module, we need the sort order.
    $config = \Drupal::config('reviews.settings');
    $sort = $config->get('reviews.sort_order') === 'newest' ? 'desc' : 'asc';

    // Get review entities for the given node.
    $review_ids = \Drupal::entityQuery('review')
      ->condition('node_id', $node->id(), '=')
      ->condition('status', 1)
      ->sort('created', $sort)
      ->accessCheck(FALSE)
      ->execute();

    if (!empty($review_ids)) {
      $entityTypeManager = ControllerBase::entityTypeManager();

      $reviews = $entityTypeManager->getStorage('review')->loadMultiple($review_ids);
      $view_builder = $entityTypeManager->getViewBuilder('review');

      $build = [];
      foreach ($reviews as $key => $review) {
        $view = $view_builder->view($review);
        $build['review_' . uniqid()] = $view;
      }
    }
    else {
      $build = [
        '#theme' => 'no_reviews',
      ];
    }

    // Add cache tags...
    $build['#cache']['tags'] = [
      'reviews:node:' . $node->id(),
    ];

    $build['#attached']['library'][] = 'reviews/reviews';

    return $build;
  }

}
