<?php

namespace Drupal\reviews\Form;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\reviews\Entity\ReviewEntity;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class ReviewForm.
 */
class ReviewForm extends FormBase {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new ReviewForm.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'review_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $nid = \Drupal::routeMatch()->getParameter('node');

    $form['nid'] = [
      '#type' => 'hidden',
      '#value' => $nid,
    ];

    $form['review_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Review Title'),
      '#description' => $this->t('Enter a title for the review.'),
      '#maxlength' => 64,
      '#size' => 64,
      '#weight' => '0',
      '#required' => TRUE,
    ];

    $form['review'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Review'),
      '#description' => $this->t('Enter your review.'),
      '#weight' => '0',
      '#required' => TRUE,
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = \Drupal::config('reviews.settings');

    // Default published status for review.
    $status = 0;

    // Load the node to see if it's bundle is the "do not moderate" list.
    $node = $this->entityTypeManager->getStorage('node')->load($form_state->getValue('nid'));
    if ($node) {
      $do_not_mod_types = $config->get('reviews.moderate_types');
      if (in_array($node->bundle(), array_filter($do_not_mod_types))) {
        $status = 1;
      }
    }

    $review = ReviewEntity::create([
      'title' => $form_state->getValue('review_title'),
      'review' => $form_state->getValue('review'),
      'status' => $status,
      'created' => \Drupal::time()->getRequestTime(),
      'user_id' => \Drupal::currentUser()->id(),
      'node_id' => $form_state->getValue('nid'),
    ]);
    $review->save();

    // Invalidate the cache tags for the local actions
    // for this module.
    Cache::invalidateTags(['reviews_action']);

    // Redirect back to the node page.
    $form_state->setRedirect(
      'entity.node.canonical',
      [
        'node' => $form_state->getValue('nid'),
      ]
    );

    // Let the user know their review has been posted.
    $this->messenger()->addMessage('Thank you for your review.');
  }

}
